!*************************************************************************
!  Driver for pes_Li2H.f90 to calculate potentail energy of Li2H molecules
!      M.A.Furudate, Chungnam National University, Korea, 2018.3.26
!------------------------------------------------------------------------
!  This is an example program to calculate potential energy of Li2H 
!  from the given Cartesian coordinates of H, Li and Li atoms.
!------------------------------------------------------------------------
!
!  Compile
!    ifort -r8 pes_Li2H.f90 driver_Li2H.f90 -o pes_Li2H.exe
!
!  Test run (coeff.dat should be placed in the same directory)
!    pes_Li2H.exe < example.inp
!
!  Input file, example.inp
!    Line 1: x,y,z coodinates of H  [a.u.]
!    Line 2: x,y,z coodinates of Li_1 [a.u.]
!    Line 3: x,y,z coodinates of Li_2 [a.u.] 
!
!  Output file, example.out
!
!------------------------------------------------------------------------
!*************************************************************************
      program main
      use pes_shell
      implicit none
!
      integer, parameter :: natom = 3 ! Number of atoms
      character(len=3),parameter :: aname(natom)=(/'H','Li1','Li2'/)
      real :: xyz(3,natom) ! input: xyz-coordinates of atoms [a.u.]
      real :: v            ! output: absolute potential energy [a.u.]
      integer :: i
      integer, save :: init=0
!
!-----------------------------------------------------------------------
! Initialization (Read the file "coeff.dat")
!-----------------------------------------------------------------------
      if (init==0)then
        call pes_init()
        init = 1
      endif
!
!-----------------------------------------------------------------------
! Input the position of atoms in Cartesian coordinate
!-----------------------------------------------------------------------
      do  i = 1,natom
      write(*,"(' x, y and z coordinates of ', a3,', [a.u.]')"),aname(i)
        read (*,*) xyz(1:3,i)
        write(*,"(3f14.8)") xyz(1:3,i)
      enddo
!
!-----------------------------------------------------------------------
! Call function to calculate potential energy
!-----------------------------------------------------------------------
      v= f(xyz)
!
      write(*,*)""
      write(*,"('Potentail energy :',f14.8,'[a.u.]')"),v

      stop
      end program main
