!******************************************************************************
! This file has three modules to generate PES of Li2H system;
!    module basis
!    module gradient
!    module pes_shell
!
! These modules are generated by MSA program;
!   Z. Xie and J.M. Bowman, J. Chem. Theory Comput. 6, 26 (2010)
! Downloaded from : https://github.com/Kee-Wang/PES-Fitting-MSA
!******************************************************************************

module basis
  implicit none

contains
  function emsav(x,c) result(v)
    implicit none
    real,dimension(1:3)::x
    real,dimension(0:251)::c
    real::v
    ! ::::::::::::::::::::
    real,dimension(0:251)::p

    call bemsav(x,p)
    v = dot_product(p,c)

    return
  end function emsav

  subroutine bemsav(x,p)
    implicit none
    real,dimension(1:3),intent(in)::x
    real,dimension(0:251),intent(out)::p
    ! ::::::::::::::::::::
    real,dimension(0:4)::m

    call evmono(x,m)
    call evpoly(m,p)

    return
  end subroutine bemsav

  subroutine evmono(x,m)
    implicit none
    real,dimension(1:3),intent(in)::x
    real,dimension(0:4),intent(out)::m
    !::::::::::::::::::::

    m(0) = 1.0D0
    m(1) = x(3)
    m(2) = x(2)
    m(3) = x(1)
    m(4) = m(2)*m(3)

    return
  end subroutine evmono

  subroutine evpoly(m,p)
    implicit none
    real,dimension(0:4),intent(in)::m
    real,dimension(0:251),intent(out)::p
    !::::::::::::::::::::

    p(0) = m(0)
    p(1) = m(1)
    p(2) = m(2) + m(3)
    p(3) = p(1)*p(2)
    p(4) = m(4)
    p(5) = p(1)*p(1)
    p(6) = p(2)*p(2) - p(4) - p(4)
    p(7) = p(1)*p(4)
    p(8) = p(1)*p(3)
    p(9) = p(1)*p(6)
    p(10) = p(4)*p(2)
    p(11) = p(1)*p(5)
    p(12) = p(2)*p(6) - p(10)
    p(13) = p(1)*p(7)
    p(14) = p(1)*p(10)
    p(15) = p(1)*p(8)
    p(16) = p(1)*p(9)
    p(17) = p(4)*p(4)
    p(18) = p(1)*p(12)
    p(19) = p(4)*p(6)
    p(20) = p(1)*p(11)
    p(21) = p(2)*p(12) - p(19)
    p(22) = p(1)*p(13)
    p(23) = p(1)*p(14)
    p(24) = p(1)*p(17)
    p(25) = p(1)*p(19)
    p(26) = p(1)*p(15)
    p(27) = p(1)*p(16)
    p(28) = p(1)*p(18)
    p(29) = p(4)*p(10)
    p(30) = p(1)*p(21)
    p(31) = p(4)*p(12)
    p(32) = p(1)*p(20)
    p(33) = p(2)*p(21) - p(31)
    p(34) = p(1)*p(22)
    p(35) = p(1)*p(23)
    p(36) = p(1)*p(24)
    p(37) = p(1)*p(25)
    p(38) = p(1)*p(29)
    p(39) = p(1)*p(31)
    p(40) = p(1)*p(26)
    p(41) = p(1)*p(27)
    p(42) = p(1)*p(28)
    p(43) = p(4)*p(17)
    p(44) = p(1)*p(30)
    p(45) = p(4)*p(19)
    p(46) = p(1)*p(33)
    p(47) = p(4)*p(21)
    p(48) = p(1)*p(32)
    p(49) = p(2)*p(33) - p(47)
    p(50) = p(1)*p(34)
    p(51) = p(1)*p(35)
    p(52) = p(1)*p(36)
    p(53) = p(1)*p(37)
    p(54) = p(1)*p(38)
    p(55) = p(1)*p(43)
    p(56) = p(1)*p(39)
    p(57) = p(1)*p(45)
    p(58) = p(1)*p(47)
    p(59) = p(1)*p(40)
    p(60) = p(1)*p(41)
    p(61) = p(1)*p(42)
    p(62) = p(1)*p(44)
    p(63) = p(4)*p(29)
    p(64) = p(1)*p(46)
    p(65) = p(4)*p(31)
    p(66) = p(1)*p(49)
    p(67) = p(4)*p(33)
    p(68) = p(1)*p(48)
    p(69) = p(2)*p(49) - p(67)
    p(70) = p(1)*p(50)
    p(71) = p(1)*p(51)
    p(72) = p(1)*p(52)
    p(73) = p(1)*p(53)
    p(74) = p(1)*p(54)
    p(75) = p(1)*p(55)
    p(76) = p(1)*p(56)
    p(77) = p(1)*p(57)
    p(78) = p(1)*p(63)
    p(79) = p(1)*p(58)
    p(80) = p(1)*p(65)
    p(81) = p(1)*p(67)
    p(82) = p(1)*p(59)
    p(83) = p(1)*p(60)
    p(84) = p(1)*p(61)
    p(85) = p(1)*p(62)
    p(86) = p(4)*p(43)
    p(87) = p(1)*p(64)
    p(88) = p(4)*p(45)
    p(89) = p(1)*p(66)
    p(90) = p(4)*p(47)
    p(91) = p(1)*p(69)
    p(92) = p(4)*p(49)
    p(93) = p(1)*p(68)
    p(94) = p(2)*p(69) - p(92)
    p(95) = p(1)*p(70)
    p(96) = p(1)*p(71)
    p(97) = p(1)*p(72)
    p(98) = p(1)*p(73)
    p(99) = p(1)*p(74)
    p(100) = p(1)*p(75)
    p(101) = p(1)*p(76)
    p(102) = p(1)*p(77)
    p(103) = p(1)*p(78)
    p(104) = p(1)*p(86)
    p(105) = p(1)*p(79)
    p(106) = p(1)*p(80)
    p(107) = p(1)*p(88)
    p(108) = p(1)*p(81)
    p(109) = p(1)*p(90)
    p(110) = p(1)*p(92)
    p(111) = p(1)*p(82)
    p(112) = p(1)*p(83)
    p(113) = p(1)*p(84)
    p(114) = p(1)*p(85)
    p(115) = p(1)*p(87)
    p(116) = p(4)*p(63)
    p(117) = p(1)*p(89)
    p(118) = p(4)*p(65)
    p(119) = p(1)*p(91)
    p(120) = p(4)*p(67)
    p(121) = p(1)*p(94)
    p(122) = p(4)*p(69)
    p(123) = p(1)*p(93)
    p(124) = p(2)*p(94) - p(122)
    p(125) = p(1)*p(95)
    p(126) = p(1)*p(96)
    p(127) = p(1)*p(97)
    p(128) = p(1)*p(98)
    p(129) = p(1)*p(99)
    p(130) = p(1)*p(100)
    p(131) = p(1)*p(101)
    p(132) = p(1)*p(102)
    p(133) = p(1)*p(103)
    p(134) = p(1)*p(104)
    p(135) = p(1)*p(105)
    p(136) = p(1)*p(106)
    p(137) = p(1)*p(107)
    p(138) = p(1)*p(116)
    p(139) = p(1)*p(108)
    p(140) = p(1)*p(109)
    p(141) = p(1)*p(118)
    p(142) = p(1)*p(110)
    p(143) = p(1)*p(120)
    p(144) = p(1)*p(122)
    p(145) = p(1)*p(111)
    p(146) = p(1)*p(112)
    p(147) = p(1)*p(113)
    p(148) = p(1)*p(114)
    p(149) = p(1)*p(115)
    p(150) = p(4)*p(86)
    p(151) = p(1)*p(117)
    p(152) = p(4)*p(88)
    p(153) = p(1)*p(119)
    p(154) = p(4)*p(90)
    p(155) = p(1)*p(121)
    p(156) = p(4)*p(92)
    p(157) = p(1)*p(124)
    p(158) = p(4)*p(94)
    p(159) = p(1)*p(123)
    p(160) = p(2)*p(124) - p(158)
    p(161) = p(1)*p(125)
    p(162) = p(1)*p(126)
    p(163) = p(1)*p(127)
    p(164) = p(1)*p(128)
    p(165) = p(1)*p(129)
    p(166) = p(1)*p(130)
    p(167) = p(1)*p(131)
    p(168) = p(1)*p(132)
    p(169) = p(1)*p(133)
    p(170) = p(1)*p(134)
    p(171) = p(1)*p(135)
    p(172) = p(1)*p(136)
    p(173) = p(1)*p(137)
    p(174) = p(1)*p(138)
    p(175) = p(1)*p(150)
    p(176) = p(1)*p(139)
    p(177) = p(1)*p(140)
    p(178) = p(1)*p(141)
    p(179) = p(1)*p(152)
    p(180) = p(1)*p(142)
    p(181) = p(1)*p(143)
    p(182) = p(1)*p(154)
    p(183) = p(1)*p(144)
    p(184) = p(1)*p(156)
    p(185) = p(1)*p(158)
    p(186) = p(1)*p(145)
    p(187) = p(1)*p(146)
    p(188) = p(1)*p(147)
    p(189) = p(1)*p(148)
    p(190) = p(1)*p(149)
    p(191) = p(1)*p(151)
    p(192) = p(4)*p(116)
    p(193) = p(1)*p(153)
    p(194) = p(4)*p(118)
    p(195) = p(1)*p(155)
    p(196) = p(4)*p(120)
    p(197) = p(1)*p(157)
    p(198) = p(4)*p(122)
    p(199) = p(1)*p(160)
    p(200) = p(4)*p(124)
    p(201) = p(1)*p(159)
    p(202) = p(2)*p(160) - p(200)
    p(203) = p(1)*p(161)
    p(204) = p(1)*p(162)
    p(205) = p(1)*p(163)
    p(206) = p(1)*p(164)
    p(207) = p(1)*p(165)
    p(208) = p(1)*p(166)
    p(209) = p(1)*p(167)
    p(210) = p(1)*p(168)
    p(211) = p(1)*p(169)
    p(212) = p(1)*p(170)
    p(213) = p(1)*p(171)
    p(214) = p(1)*p(172)
    p(215) = p(1)*p(173)
    p(216) = p(1)*p(174)
    p(217) = p(1)*p(175)
    p(218) = p(1)*p(176)
    p(219) = p(1)*p(177)
    p(220) = p(1)*p(178)
    p(221) = p(1)*p(179)
    p(222) = p(1)*p(192)
    p(223) = p(1)*p(180)
    p(224) = p(1)*p(181)
    p(225) = p(1)*p(182)
    p(226) = p(1)*p(194)
    p(227) = p(1)*p(183)
    p(228) = p(1)*p(184)
    p(229) = p(1)*p(196)
    p(230) = p(1)*p(185)
    p(231) = p(1)*p(198)
    p(232) = p(1)*p(200)
    p(233) = p(1)*p(186)
    p(234) = p(1)*p(187)
    p(235) = p(1)*p(188)
    p(236) = p(1)*p(189)
    p(237) = p(1)*p(190)
    p(238) = p(1)*p(191)
    p(239) = p(4)*p(150)
    p(240) = p(1)*p(193)
    p(241) = p(4)*p(152)
    p(242) = p(1)*p(195)
    p(243) = p(4)*p(154)
    p(244) = p(1)*p(197)
    p(245) = p(4)*p(156)
    p(246) = p(1)*p(199)
    p(247) = p(4)*p(158)
    p(248) = p(1)*p(202)
    p(249) = p(4)*p(160)
    p(250) = p(1)*p(201)
    p(251) = p(2)*p(202) - p(249)

    return
  end subroutine evpoly

end module basis

module gradient
  implicit none

contains
  function demsav(drdx,c,m,p,flag) result(grad)
    implicit none
    real,dimension(9,3)::drdx
    real,dimension(0:251)::c
    real,dimension(0:4)::m
    real,dimension(0:251)::p
    real::grad
    integer::flag
    ! ::::::::::::::::::::
    real,dimension(0:251)::dp

    call dbemsav(drdx,dp,m,p,flag)
    grad = dot_product(dp,c)

    return
  end function demsav

  subroutine dbemsav(drdx,dp,m,p,flag)
    implicit none
    real,dimension(9,3),intent(in)::drdx
    real,dimension(0:251),intent(out)::dp
    real,dimension(0:4),intent(in)::m
    real,dimension(0:251),intent(in)::p
    integer::flag
    ! ::::::::::::::::::::
    real,dimension(0:4)::dm

    call devmono(drdx,dm,m,flag)
    call devpoly(dm,p,dp)

    return
  end subroutine dbemsav

  subroutine devmono(drdx,dm,m,flag)
    implicit none
    real,dimension(9,3),intent(in)::drdx
    real,dimension(0:4),intent(out)::dm
    real,dimension(0:4),intent(in)::m
    integer::flag
    !::::::::::::::::::::
    real::a

    a = 3.0d0

    dm(0) = 0.0D0
    dm(1) = -m(1)/a*drdx(flag,3)
    dm(2) = -m(2)/a*drdx(flag,2)
    dm(3) = -m(3)/a*drdx(flag,1)
    dm(4) = dm(2)*m(3) + m(2)*dm(3)

    return
  end subroutine devmono

  subroutine devpoly(dm,p,dp)
    implicit none
    real,dimension(0:4),intent(in)::dm
    real,dimension(0:251),intent(in)::p
    real,dimension(0:251),intent(out)::dp
    !::::::::::::::::::::

    dp(0) = dm(0)
    dp(1) = dm(1)
    dp(2) = dm(2) + dm(3)
    dp(3) = dp(1)*p(2) + p(1)*dp(2)
    dp(4) = dm(4)
    dp(5) = dp(1)*p(1) + p(1)*dp(1)
    dp(6) = dp(2)*p(2) + p(2)*dp(2) - dp(4) - dp(4)
    dp(7) = dp(1)*p(4) + p(1)*dp(4)
    dp(8) = dp(1)*p(3) + p(1)*dp(3)
    dp(9) = dp(1)*p(6) + p(1)*dp(6)
    dp(10) = dp(4)*p(2) + p(4)*dp(2)
    dp(11) = dp(1)*p(5) + p(1)*dp(5)
    dp(12) = dp(2)*p(6) + p(2)*dp(6) - dp(10)
    dp(13) = dp(1)*p(7) + p(1)*dp(7)
    dp(14) = dp(1)*p(10) + p(1)*dp(10)
    dp(15) = dp(1)*p(8) + p(1)*dp(8)
    dp(16) = dp(1)*p(9) + p(1)*dp(9)
    dp(17) = dp(4)*p(4) + p(4)*dp(4)
    dp(18) = dp(1)*p(12) + p(1)*dp(12)
    dp(19) = dp(4)*p(6) + p(4)*dp(6)
    dp(20) = dp(1)*p(11) + p(1)*dp(11)
    dp(21) = dp(2)*p(12) + p(2)*dp(12) - dp(19)
    dp(22) = dp(1)*p(13) + p(1)*dp(13)
    dp(23) = dp(1)*p(14) + p(1)*dp(14)
    dp(24) = dp(1)*p(17) + p(1)*dp(17)
    dp(25) = dp(1)*p(19) + p(1)*dp(19)
    dp(26) = dp(1)*p(15) + p(1)*dp(15)
    dp(27) = dp(1)*p(16) + p(1)*dp(16)
    dp(28) = dp(1)*p(18) + p(1)*dp(18)
    dp(29) = dp(4)*p(10) + p(4)*dp(10)
    dp(30) = dp(1)*p(21) + p(1)*dp(21)
    dp(31) = dp(4)*p(12) + p(4)*dp(12)
    dp(32) = dp(1)*p(20) + p(1)*dp(20)
    dp(33) = dp(2)*p(21) + p(2)*dp(21) - dp(31)
    dp(34) = dp(1)*p(22) + p(1)*dp(22)
    dp(35) = dp(1)*p(23) + p(1)*dp(23)
    dp(36) = dp(1)*p(24) + p(1)*dp(24)
    dp(37) = dp(1)*p(25) + p(1)*dp(25)
    dp(38) = dp(1)*p(29) + p(1)*dp(29)
    dp(39) = dp(1)*p(31) + p(1)*dp(31)
    dp(40) = dp(1)*p(26) + p(1)*dp(26)
    dp(41) = dp(1)*p(27) + p(1)*dp(27)
    dp(42) = dp(1)*p(28) + p(1)*dp(28)
    dp(43) = dp(4)*p(17) + p(4)*dp(17)
    dp(44) = dp(1)*p(30) + p(1)*dp(30)
    dp(45) = dp(4)*p(19) + p(4)*dp(19)
    dp(46) = dp(1)*p(33) + p(1)*dp(33)
    dp(47) = dp(4)*p(21) + p(4)*dp(21)
    dp(48) = dp(1)*p(32) + p(1)*dp(32)
    dp(49) = dp(2)*p(33) + p(2)*dp(33) - dp(47)
    dp(50) = dp(1)*p(34) + p(1)*dp(34)
    dp(51) = dp(1)*p(35) + p(1)*dp(35)
    dp(52) = dp(1)*p(36) + p(1)*dp(36)
    dp(53) = dp(1)*p(37) + p(1)*dp(37)
    dp(54) = dp(1)*p(38) + p(1)*dp(38)
    dp(55) = dp(1)*p(43) + p(1)*dp(43)
    dp(56) = dp(1)*p(39) + p(1)*dp(39)
    dp(57) = dp(1)*p(45) + p(1)*dp(45)
    dp(58) = dp(1)*p(47) + p(1)*dp(47)
    dp(59) = dp(1)*p(40) + p(1)*dp(40)
    dp(60) = dp(1)*p(41) + p(1)*dp(41)
    dp(61) = dp(1)*p(42) + p(1)*dp(42)
    dp(62) = dp(1)*p(44) + p(1)*dp(44)
    dp(63) = dp(4)*p(29) + p(4)*dp(29)
    dp(64) = dp(1)*p(46) + p(1)*dp(46)
    dp(65) = dp(4)*p(31) + p(4)*dp(31)
    dp(66) = dp(1)*p(49) + p(1)*dp(49)
    dp(67) = dp(4)*p(33) + p(4)*dp(33)
    dp(68) = dp(1)*p(48) + p(1)*dp(48)
    dp(69) = dp(2)*p(49) + p(2)*dp(49) - dp(67)
    dp(70) = dp(1)*p(50) + p(1)*dp(50)
    dp(71) = dp(1)*p(51) + p(1)*dp(51)
    dp(72) = dp(1)*p(52) + p(1)*dp(52)
    dp(73) = dp(1)*p(53) + p(1)*dp(53)
    dp(74) = dp(1)*p(54) + p(1)*dp(54)
    dp(75) = dp(1)*p(55) + p(1)*dp(55)
    dp(76) = dp(1)*p(56) + p(1)*dp(56)
    dp(77) = dp(1)*p(57) + p(1)*dp(57)
    dp(78) = dp(1)*p(63) + p(1)*dp(63)
    dp(79) = dp(1)*p(58) + p(1)*dp(58)
    dp(80) = dp(1)*p(65) + p(1)*dp(65)
    dp(81) = dp(1)*p(67) + p(1)*dp(67)
    dp(82) = dp(1)*p(59) + p(1)*dp(59)
    dp(83) = dp(1)*p(60) + p(1)*dp(60)
    dp(84) = dp(1)*p(61) + p(1)*dp(61)
    dp(85) = dp(1)*p(62) + p(1)*dp(62)
    dp(86) = dp(4)*p(43) + p(4)*dp(43)
    dp(87) = dp(1)*p(64) + p(1)*dp(64)
    dp(88) = dp(4)*p(45) + p(4)*dp(45)
    dp(89) = dp(1)*p(66) + p(1)*dp(66)
    dp(90) = dp(4)*p(47) + p(4)*dp(47)
    dp(91) = dp(1)*p(69) + p(1)*dp(69)
    dp(92) = dp(4)*p(49) + p(4)*dp(49)
    dp(93) = dp(1)*p(68) + p(1)*dp(68)
    dp(94) = dp(2)*p(69) + p(2)*dp(69) - dp(92)
    dp(95) = dp(1)*p(70) + p(1)*dp(70)
    dp(96) = dp(1)*p(71) + p(1)*dp(71)
    dp(97) = dp(1)*p(72) + p(1)*dp(72)
    dp(98) = dp(1)*p(73) + p(1)*dp(73)
    dp(99) = dp(1)*p(74) + p(1)*dp(74)
    dp(100) = dp(1)*p(75) + p(1)*dp(75)
    dp(101) = dp(1)*p(76) + p(1)*dp(76)
    dp(102) = dp(1)*p(77) + p(1)*dp(77)
    dp(103) = dp(1)*p(78) + p(1)*dp(78)
    dp(104) = dp(1)*p(86) + p(1)*dp(86)
    dp(105) = dp(1)*p(79) + p(1)*dp(79)
    dp(106) = dp(1)*p(80) + p(1)*dp(80)
    dp(107) = dp(1)*p(88) + p(1)*dp(88)
    dp(108) = dp(1)*p(81) + p(1)*dp(81)
    dp(109) = dp(1)*p(90) + p(1)*dp(90)
    dp(110) = dp(1)*p(92) + p(1)*dp(92)
    dp(111) = dp(1)*p(82) + p(1)*dp(82)
    dp(112) = dp(1)*p(83) + p(1)*dp(83)
    dp(113) = dp(1)*p(84) + p(1)*dp(84)
    dp(114) = dp(1)*p(85) + p(1)*dp(85)
    dp(115) = dp(1)*p(87) + p(1)*dp(87)
    dp(116) = dp(4)*p(63) + p(4)*dp(63)
    dp(117) = dp(1)*p(89) + p(1)*dp(89)
    dp(118) = dp(4)*p(65) + p(4)*dp(65)
    dp(119) = dp(1)*p(91) + p(1)*dp(91)
    dp(120) = dp(4)*p(67) + p(4)*dp(67)
    dp(121) = dp(1)*p(94) + p(1)*dp(94)
    dp(122) = dp(4)*p(69) + p(4)*dp(69)
    dp(123) = dp(1)*p(93) + p(1)*dp(93)
    dp(124) = dp(2)*p(94) + p(2)*dp(94) - dp(122)
    dp(125) = dp(1)*p(95) + p(1)*dp(95)
    dp(126) = dp(1)*p(96) + p(1)*dp(96)
    dp(127) = dp(1)*p(97) + p(1)*dp(97)
    dp(128) = dp(1)*p(98) + p(1)*dp(98)
    dp(129) = dp(1)*p(99) + p(1)*dp(99)
    dp(130) = dp(1)*p(100) + p(1)*dp(100)
    dp(131) = dp(1)*p(101) + p(1)*dp(101)
    dp(132) = dp(1)*p(102) + p(1)*dp(102)
    dp(133) = dp(1)*p(103) + p(1)*dp(103)
    dp(134) = dp(1)*p(104) + p(1)*dp(104)
    dp(135) = dp(1)*p(105) + p(1)*dp(105)
    dp(136) = dp(1)*p(106) + p(1)*dp(106)
    dp(137) = dp(1)*p(107) + p(1)*dp(107)
    dp(138) = dp(1)*p(116) + p(1)*dp(116)
    dp(139) = dp(1)*p(108) + p(1)*dp(108)
    dp(140) = dp(1)*p(109) + p(1)*dp(109)
    dp(141) = dp(1)*p(118) + p(1)*dp(118)
    dp(142) = dp(1)*p(110) + p(1)*dp(110)
    dp(143) = dp(1)*p(120) + p(1)*dp(120)
    dp(144) = dp(1)*p(122) + p(1)*dp(122)
    dp(145) = dp(1)*p(111) + p(1)*dp(111)
    dp(146) = dp(1)*p(112) + p(1)*dp(112)
    dp(147) = dp(1)*p(113) + p(1)*dp(113)
    dp(148) = dp(1)*p(114) + p(1)*dp(114)
    dp(149) = dp(1)*p(115) + p(1)*dp(115)
    dp(150) = dp(4)*p(86) + p(4)*dp(86)
    dp(151) = dp(1)*p(117) + p(1)*dp(117)
    dp(152) = dp(4)*p(88) + p(4)*dp(88)
    dp(153) = dp(1)*p(119) + p(1)*dp(119)
    dp(154) = dp(4)*p(90) + p(4)*dp(90)
    dp(155) = dp(1)*p(121) + p(1)*dp(121)
    dp(156) = dp(4)*p(92) + p(4)*dp(92)
    dp(157) = dp(1)*p(124) + p(1)*dp(124)
    dp(158) = dp(4)*p(94) + p(4)*dp(94)
    dp(159) = dp(1)*p(123) + p(1)*dp(123)
    dp(160) = dp(2)*p(124) + p(2)*dp(124) - dp(158)
    dp(161) = dp(1)*p(125) + p(1)*dp(125)
    dp(162) = dp(1)*p(126) + p(1)*dp(126)
    dp(163) = dp(1)*p(127) + p(1)*dp(127)
    dp(164) = dp(1)*p(128) + p(1)*dp(128)
    dp(165) = dp(1)*p(129) + p(1)*dp(129)
    dp(166) = dp(1)*p(130) + p(1)*dp(130)
    dp(167) = dp(1)*p(131) + p(1)*dp(131)
    dp(168) = dp(1)*p(132) + p(1)*dp(132)
    dp(169) = dp(1)*p(133) + p(1)*dp(133)
    dp(170) = dp(1)*p(134) + p(1)*dp(134)
    dp(171) = dp(1)*p(135) + p(1)*dp(135)
    dp(172) = dp(1)*p(136) + p(1)*dp(136)
    dp(173) = dp(1)*p(137) + p(1)*dp(137)
    dp(174) = dp(1)*p(138) + p(1)*dp(138)
    dp(175) = dp(1)*p(150) + p(1)*dp(150)
    dp(176) = dp(1)*p(139) + p(1)*dp(139)
    dp(177) = dp(1)*p(140) + p(1)*dp(140)
    dp(178) = dp(1)*p(141) + p(1)*dp(141)
    dp(179) = dp(1)*p(152) + p(1)*dp(152)
    dp(180) = dp(1)*p(142) + p(1)*dp(142)
    dp(181) = dp(1)*p(143) + p(1)*dp(143)
    dp(182) = dp(1)*p(154) + p(1)*dp(154)
    dp(183) = dp(1)*p(144) + p(1)*dp(144)
    dp(184) = dp(1)*p(156) + p(1)*dp(156)
    dp(185) = dp(1)*p(158) + p(1)*dp(158)
    dp(186) = dp(1)*p(145) + p(1)*dp(145)
    dp(187) = dp(1)*p(146) + p(1)*dp(146)
    dp(188) = dp(1)*p(147) + p(1)*dp(147)
    dp(189) = dp(1)*p(148) + p(1)*dp(148)
    dp(190) = dp(1)*p(149) + p(1)*dp(149)
    dp(191) = dp(1)*p(151) + p(1)*dp(151)
    dp(192) = dp(4)*p(116) + p(4)*dp(116)
    dp(193) = dp(1)*p(153) + p(1)*dp(153)
    dp(194) = dp(4)*p(118) + p(4)*dp(118)
    dp(195) = dp(1)*p(155) + p(1)*dp(155)
    dp(196) = dp(4)*p(120) + p(4)*dp(120)
    dp(197) = dp(1)*p(157) + p(1)*dp(157)
    dp(198) = dp(4)*p(122) + p(4)*dp(122)
    dp(199) = dp(1)*p(160) + p(1)*dp(160)
    dp(200) = dp(4)*p(124) + p(4)*dp(124)
    dp(201) = dp(1)*p(159) + p(1)*dp(159)
    dp(202) = dp(2)*p(160) + p(2)*dp(160) - dp(200)
    dp(203) = dp(1)*p(161) + p(1)*dp(161)
    dp(204) = dp(1)*p(162) + p(1)*dp(162)
    dp(205) = dp(1)*p(163) + p(1)*dp(163)
    dp(206) = dp(1)*p(164) + p(1)*dp(164)
    dp(207) = dp(1)*p(165) + p(1)*dp(165)
    dp(208) = dp(1)*p(166) + p(1)*dp(166)
    dp(209) = dp(1)*p(167) + p(1)*dp(167)
    dp(210) = dp(1)*p(168) + p(1)*dp(168)
    dp(211) = dp(1)*p(169) + p(1)*dp(169)
    dp(212) = dp(1)*p(170) + p(1)*dp(170)
    dp(213) = dp(1)*p(171) + p(1)*dp(171)
    dp(214) = dp(1)*p(172) + p(1)*dp(172)
    dp(215) = dp(1)*p(173) + p(1)*dp(173)
    dp(216) = dp(1)*p(174) + p(1)*dp(174)
    dp(217) = dp(1)*p(175) + p(1)*dp(175)
    dp(218) = dp(1)*p(176) + p(1)*dp(176)
    dp(219) = dp(1)*p(177) + p(1)*dp(177)
    dp(220) = dp(1)*p(178) + p(1)*dp(178)
    dp(221) = dp(1)*p(179) + p(1)*dp(179)
    dp(222) = dp(1)*p(192) + p(1)*dp(192)
    dp(223) = dp(1)*p(180) + p(1)*dp(180)
    dp(224) = dp(1)*p(181) + p(1)*dp(181)
    dp(225) = dp(1)*p(182) + p(1)*dp(182)
    dp(226) = dp(1)*p(194) + p(1)*dp(194)
    dp(227) = dp(1)*p(183) + p(1)*dp(183)
    dp(228) = dp(1)*p(184) + p(1)*dp(184)
    dp(229) = dp(1)*p(196) + p(1)*dp(196)
    dp(230) = dp(1)*p(185) + p(1)*dp(185)
    dp(231) = dp(1)*p(198) + p(1)*dp(198)
    dp(232) = dp(1)*p(200) + p(1)*dp(200)
    dp(233) = dp(1)*p(186) + p(1)*dp(186)
    dp(234) = dp(1)*p(187) + p(1)*dp(187)
    dp(235) = dp(1)*p(188) + p(1)*dp(188)
    dp(236) = dp(1)*p(189) + p(1)*dp(189)
    dp(237) = dp(1)*p(190) + p(1)*dp(190)
    dp(238) = dp(1)*p(191) + p(1)*dp(191)
    dp(239) = dp(4)*p(150) + p(4)*dp(150)
    dp(240) = dp(1)*p(193) + p(1)*dp(193)
    dp(241) = dp(4)*p(152) + p(4)*dp(152)
    dp(242) = dp(1)*p(195) + p(1)*dp(195)
    dp(243) = dp(4)*p(154) + p(4)*dp(154)
    dp(244) = dp(1)*p(197) + p(1)*dp(197)
    dp(245) = dp(4)*p(156) + p(4)*dp(156)
    dp(246) = dp(1)*p(199) + p(1)*dp(199)
    dp(247) = dp(4)*p(158) + p(4)*dp(158)
    dp(248) = dp(1)*p(202) + p(1)*dp(202)
    dp(249) = dp(4)*p(160) + p(4)*dp(160)
    dp(250) = dp(1)*p(201) + p(1)*dp(201)
    dp(251) = dp(2)*p(202) + p(2)*dp(202) - dp(249)

    return
  end subroutine devpoly

end module gradient

module pes_shell
use basis
use gradient
implicit none

  real::coeff(1:252) ! change to number of coefficients
                     ! (size of c in bemsa.f90)
  save coeff

contains
  !==================================!
  ! read the coefficients of the PES !
  !==================================!
  subroutine pes_init()
    !::::::::::::::::::
    integer::i

    open(10,file='coeff.dat',status='old')

    do i=1,size(coeff)
       read (10,*) coeff(i)
    end do

    return
    close (10)
  end subroutine pes_init

  !====================================!
  ! Function to evaluate the potential !
  !====================================!
  function f(xyz)
    real,dimension(:,:),intent(in)::xyz
    real::f
    !::::::::::::::::::::::::::::::
    real,dimension(size(xyz,2)*(size(xyz,2)-1)/2)::x
    real,dimension(3)::dr
    real::a0  ! the same as the fitting code
    integer::i,j,k

    a0 = 3.0d0

    k = 1
    do i=1,size(xyz,2)-1
       do j=i+1,size(xyz,2)
          dr = xyz(:,i) - xyz(:,j)
          x(k) = sqrt(dot_product(dr,dr))
          k = k+1
       end do
    end do

    do i=1,size(x)
       x(i)=exp(-x(i)/a0)
    end do

    f=emsav(x,coeff)

    return
  end function f

  !===========================!
  ! function to calculate the !
  ! analytical gradient       !
  !===========================!
  function g(xyz)
    real,dimension(:,:),intent(in)::xyz
    real,dimension(size(xyz,2)*3)::g
    !::::::::::::::::::::
    real,dimension(size(xyz,2)*(size(xyz,2)-1)/2)::r,x
    real,dimension(3,size(xyz,2)*(size(xyz,2)-1)/2)::dr
    real,dimension(size(xyz,2)*3,size(xyz,2)*(size(xyz,2)-1)/2)::drdx
    real,dimension(1:252)::p   ! change to number of popynomials
                               ! (size of p in bemsa.f90)
    real,dimension(1:5)::m  ! change to number of monomials
                              ! (size of m in bemsa.f90
    real::a0
    integer::i,j,k

    a0 = 3.0d0

    k = 1
    drdx = 0.d0
    do i=1,size(xyz,2)-1
       do j=i+1,size(xyz,2)
          dr(:,k) = xyz(:,i) - xyz(:,j)
          r(k) = sqrt(dot_product(dr(:,k),dr(:,k)))

          drdx(3*i-2:3*i,k) = dr(:,k)/r(k)
          drdx(3*j-2:3*j,k) = -drdx(3*i-2:3*i,k)
          k = k+1
       end do
    end do

    do i=1,size(x)
       x(i)=exp(-r(i)/a0)
    end do

    call evmono(x,m)
    call evpoly(m,p)

    do i=1,3*size(xyz,2)
       g(i) = demsav(drdx,coeff,m,p,i)
    end do

    return
  end function g

end module pes_shell
